@@ -1,13 +1,13 @@
 
 setMethod("length", signature(x="SpatRasterDataset"),
 	function(x) {
-		x@ptr$nsds()
+		x@pntr$nsds()
 	}
 )
 
 
 setMethod("sds", signature(x="character"),
-	function(x, ids=0, opts=NULL, raw=FALSE) {
+	function(x, ids=0, opts=NULL, raw=FALSE, domains="") {
 
 		if (length(x) > 1) {
 			r <- lapply(x, rast, opts=opts, raw=raw)
@@ -29,8 +29,10 @@ setMethod("sds", signature(x="character"),
 			useids <- TRUE
 		}
 		if (is.null(opts)) opts <- ""[0]
-		if (raw) opts <- c(opts, "so=false")		
-		r@ptr <- SpatRasterStack$new(f, ids, useids, opts)
+		if (raw) opts <- c(opts, "so=false")
+		
+		domains <- clean_domains(domains)		
+		r@pntr <- SpatRasterStack$new(f, ids, useids, opts, domains)
 		messages(r, "sds")
 	}
 )
@@ -38,7 +40,7 @@ setMethod("sds", signature(x="character"),
 setMethod("sds", signature(x="missing"),
 	function(x) {
 		r <- methods::new("SpatRasterDataset")
-		r@ptr <- SpatRasterStack$new()
+		r@pntr <- SpatRasterStack$new()
 		r
 	}
 )
@@ -47,8 +49,8 @@ setMethod("sds", signature(x="missing"),
 setMethod("sds", signature(x="SpatRaster"),
 	function(x, ...) {
 		r <- methods::new("SpatRasterDataset")
-		r@ptr <- SpatRasterStack$new()
-		r@ptr$add(x@ptr, varnames(x)[1], longnames(x)[1], units(x)[1], FALSE)
+		r@pntr <- SpatRasterStack$new()
+		r@pntr$add(x@pntr, varnames(x)[1], longnames(x)[1], units(x)[1], FALSE)
 		dots <- list(...)
 		nms <- names(dots)
 		if (is.null(nms)) nms = ""
@@ -57,7 +59,7 @@ setMethod("sds", signature(x="SpatRaster"),
 			if (inherits(dots[[i]], "SpatRaster")) {
 				vname <- nms[i]
 				if (vname == "") vname = varnames(dots[[i]])[1]
-				r@ptr$add(dots[[i]]@ptr, vname, longnames(dots[[i]])[1], units(dots[[i]])[1], FALSE)
+				r@pntr$add(dots[[i]]@pntr, vname, longnames(dots[[i]])[1], units(dots[[i]])[1], FALSE)
 			}
 		}
 		messages(r, "sds")
@@ -67,16 +69,16 @@ setMethod("sds", signature(x="SpatRaster"),
 setMethod("sds", signature(x="list"),
 	function(x) {
 		r <- methods::new("SpatRasterDataset")
-		r@ptr <- SpatRasterStack$new()
+		r@pntr <- SpatRasterStack$new()
 		nms <- names(x)
 		if (is.null(nms)) nms <- rep("", length(x))
 		for (i in seq_along(x)) {
 			if (inherits(x[[i]], "SpatRaster")) {
-				r@ptr$add(x[[i]]@ptr, nms[i], "", "", FALSE)
+				r@pntr$add(x[[i]]@pntr, nms[i], "", "", FALSE)
 			} else if (inherits(x[[i]], "SpatRasterDataset")) {
 				y <- as.list(x[[i]])
 				ynms <- names(x[[i]])
-				s <- sapply(y, function(j) r@ptr$add(j@ptr, ynms[j], "", "", FALSE))
+				s <- sapply(y, function(j) r@pntr$add(j@pntr, ynms[j], "", "", FALSE))
 			} else {
 				name <- names(x[[i]])
 				cls <- paste(class(x[[i]]), collapse=", ")
@@ -139,7 +141,7 @@ setMethod("sds", signature(x="stars_proxy"),
 setMethod("c", signature(x="SpatRasterDataset"),
 	function(x, ...) {
 
-		x@ptr <- x@ptr$subset((1:x@ptr$nsds()) -1 ) # why? make a copy?
+		x@pntr <- x@pntr$subset((1:x@pntr$nsds()) -1 ) # why? make a copy?
 
 		dots <- list(...)
 		nms <- names(dots)
@@ -148,14 +150,14 @@ setMethod("c", signature(x="SpatRasterDataset"),
 			if (inherits(dots[[i]], "SpatRasterDataset")) {
 				sdsnms <- names(dots[[i]])
 				for (j in 1:(length(dots[[i]]))) {
-					if (!x@ptr$add(dots[[i]][[j]]@ptr, sdsnms[j], "", "", FALSE)) {
+					if (!x@pntr$add(dots[[i]][[j]]@pntr, sdsnms[j], "", "", FALSE)) {
 						messages(x, "c")
 					}
 				}
 
 			} else if (inherits(dots[[i]], "SpatRaster")) {
 				if (is.null(nms)) error("c", "arguments must be named")
-				if (!x@ptr$add(dots[[i]]@ptr, nms[i], "", "", FALSE)) {
+				if (!x@pntr$add(dots[[i]]@pntr, nms[i], "", "", FALSE)) {
 					messages(x, "c")
 				}
 			} else {
@@ -176,9 +178,9 @@ setReplaceMethod("[", c("SpatRasterDataset", "numeric", "missing"),
 		i <- sort(i)
 		for (j in i) {
 			if (j == (length(x)+1)) {
-				x@ptr$add(value@ptr, "", "", "", FALSE)
+				x@pntr$add(value@pntr, "", "", "", FALSE)
 			} else {
-				x@ptr$replace(j-1, value@ptr, FALSE)
+				x@pntr$replace(j-1, value@pntr, FALSE)
 			}
 		}
 		messages(x, "`[`")
@@ -191,11 +193,11 @@ function(x, i, j, drop=TRUE) {
 	i <- positive_indices(i, length(x), TRUE, "`[`(i)")
 
 	if (drop && (length(i) == 1)) {
-		ptr <- x@ptr$getsds(i-1)
+		tptr <- x@pntr$getsds(i-1)
 		x <- rast()
-		x@ptr <- ptr
+		x@pntr <- tptr
 	} else {
-		x@ptr <- x@ptr$subset(i-1)
+		x@pntr <- x@pntr$subset(i-1)
 	}
 	messages(x, "`[`")
 })
@@ -210,10 +212,10 @@ function(x, i, j, drop=TRUE) {
 		out <- rast(out)
 	} else {
 		out <- sds()
-		nms <- x@ptr$names
+		nms <- x@pntr$names
 		for (k in nd) {
 			r <- x[k][[j]]
-			out@ptr$add(r@ptr, nms[k], "", "", FALSE)
+			out@pntr$add(r@pntr, nms[k], "", "", FALSE)
 		}
 	}
 	messages(out, "`[`")
@@ -227,13 +229,13 @@ function(x, i, j, drop=TRUE) {
 
 setMethod("[", c("SpatRasterDataset", "missing", "numeric"),
 function(x, i, j, drop=TRUE) {
-	`[`(x, i=1:x@ptr$nsds(), j=j, drop=drop)
+	`[`(x, i=1:x@pntr$nsds(), j=j, drop=drop)
 })
 
 setMethod("[", c("SpatRasterDataset", "missing", "logical"),
 function(x, i, j, drop=TRUE) {
 	j <- positive_indices(j, min(nlyr(x)))
-	`[`(x, i=1:x@ptr$nsds(), j=j, drop=drop)
+	`[`(x, i=1:x@pntr$nsds(), j=j, drop=drop)
 })
 
 
@@ -285,7 +287,7 @@ setMethod("$", "SpatRasterDataset",
 setMethod("sprc", signature(x="missing"),
 	function(x) {
 		r <- methods::new("SpatRasterCollection")
-		r@ptr <- SpatRasterCollection$new()
+		r@pntr <- SpatRasterCollection$new()
 		r
 	}
 )
@@ -301,15 +303,15 @@ setMethod("sprc", signature(x="SpatRaster"),
 setMethod("sprc", signature(x="list"),
 	function(x) {
 		n <- length(x)
-		ptr <- SpatRasterCollection$new()
+		tptr <- SpatRasterCollection$new()
 		if (n > 0) {
 			for (i in 1:n) {
 				if (inherits(x[[i]], "SpatRaster")) {
-					ptr$add(x[[i]]@ptr, "")
+					tptr$add(x[[i]]@pntr, "")
 				} else if (inherits(x[[i]], "SpatRasterCollection") | 
 							inherits(x[[i]], "SpatRasterDataset")) {
 					y <- as.list(x[[i]])
-					s <- sapply(y, function(j) ptr$add(j@ptr, ""))
+					s <- sapply(y, function(j) tptr$add(j@pntr, ""))
 				} else {
 					name <- names(x[[i]])
 					cls <- paste(class(x[[i]]), collapse=", ")
@@ -318,14 +320,14 @@ setMethod("sprc", signature(x="list"),
 			}
 		}
 		r <- new("SpatRasterCollection")
-		r@ptr <- ptr
+		r@pntr <- tptr
 		if (length(r) == length(x)) names(r) <- names(x)
 		r
 	}
 )
 
 setMethod("sprc", signature(x="character"),
-	function(x, ids=0, opts=NULL, raw=FALSE) {
+	function(x, ids=0, opts=NULL, raw=FALSE, domains="") {
 
 		if (length(x) > 1) {
 			r <- lapply(x, rast)
@@ -349,7 +351,8 @@ setMethod("sprc", signature(x="character"),
 		if (is.null(opts)) opts <- ""[0]
 		if (raw) opts <- c(opts, "so=false")		
 		
-		r@ptr <- SpatRasterCollection$new(f, ids, useids, opts)
+		domains <- clean_domains(domains)
+		r@pntr <- SpatRasterCollection$new(f, ids, useids, opts, domains)
 		messages(r, "sprc")
 	}
 )
@@ -357,7 +360,7 @@ setMethod("sprc", signature(x="character"),
 
 setMethod("length", signature(x="SpatRasterCollection"),
 	function(x) {
-		x@ptr$length()
+		x@pntr$length()
 	}
 )
 
@@ -365,16 +368,16 @@ setMethod("[", c("SpatRasterCollection", "numeric", "missing"),
 function(x, i, j, ... ,drop=TRUE) {
 	i <- positive_indices(i, length(x), TRUE, "`[`(i)")
 	if (drop && (length(i) == 1)) {
-		ptr <- x@ptr$x[[i]]
+		tptr <- x@pntr$x[[i]]
 		x <- rast()
-		x@ptr <- ptr
+		x@pntr <- tptr
 	} else {
-		s <- x@ptr$x[i]
-		ptr <- SpatRasterCollection$new()
+		s <- x@pntr$x[i]
+		tptr <- SpatRasterCollection$new()
 		for (i in 1:length(s)) {
-			ptr$add(s[[i]], "")
+			tptr$add(s[[i]], "")
 		}
-		x@ptr <- ptr
+		x@pntr <- tptr
 	}
 	messages(x, "`[`")
 })
@@ -382,7 +385,7 @@ function(x, i, j, ... ,drop=TRUE) {
 
 setMethod("add<-", signature("SpatRasterCollection", "SpatRaster"),
 	function(x, value) {
-		x@ptr$add(value@ptr, "")
+		x@pntr$add(value@pntr, "")
 		messages(x, "add")
 	}
 )
@@ -390,7 +393,7 @@ setMethod("add<-", signature("SpatRasterCollection", "SpatRaster"),
 
 setMethod("add<-", signature("SpatRasterDataset", "SpatRaster"),
 	function(x, value) {
-		x@ptr$add(value@ptr, "")
+		x@pntr$add(value@pntr, "")
 		messages(x, "add")
 	}
 )
@@ -407,7 +410,7 @@ setMethod("c", signature(x="SpatRasterCollection"),
 setMethod("c", signature(x="SpatRasterCollection"),
 	function(x, ...) {
 
-		x@ptr <- x@ptr$deepcopy()
+		x@pntr <- x@pntr$deepcopy()
 
 		dots <- list(...)
 		nms <- names(dots)
@@ -415,7 +418,7 @@ setMethod("c", signature(x="SpatRasterCollection"),
 			if (inherits(dots[[i]], "SpatRasterCollection") | inherits(dots[[i]], "SpatRasterDataset")) {
 				sdsnms <- names(dots[[i]])
 				for (j in 1:(length(dots[[i]]))) {
-					x@ptr$add(dots[[i]][[j]]@ptr, sdsnms[j])
+					x@pntr$add(dots[[i]][[j]]@pntr, sdsnms[j])
 				}
 			} else if (inherits(dots[[i]], "SpatRaster")) {
 				if (is.null(nms)) { 
@@ -423,7 +426,7 @@ setMethod("c", signature(x="SpatRasterCollection"),
 				} else { 
 					name <- nms[i] 
 				}
-				x@ptr$add(dots[[i]]@ptr, name)
+				x@pntr$add(dots[[i]]@pntr, name)
 			} else {
 				error("c", "arguments must be SpatRaster, SpatRasterCollection, or SpatRasterDataset")
 			}
