@@ -3,7 +3,6 @@
 # Version 1.0
 # License GPL v3
 
-
 setMethod("buffer", signature(x="SpatRaster"),
 	function(x, width, background=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
@@ -39,12 +38,11 @@ setMethod("buffer", signature(x="SpatRaster"),
 #
 #)
 
-setMethod("distance", signature(x="SpatRaster", y="missing"),
-	function(x, y, target=NA, exclude=NULL, unit="m", method="haversine", filename="", ...) {
 
-		if (!(method %in% c("geo", "haversine", "cosine"))) {
-			error("distance", "not a valid method. Should be one of: 'geo', 'haversine', 'cosine'")
-		}
+setMethod("distance", signature(x="SpatRaster", y="missing"), 
+	function(x, y, target=NA, exclude=NULL, unit="m", method="haversine", maxdist=NA, values=FALSE, filename="", ...) {
+
+		method <- match.arg(tolower(method), c("cosine", "haversine", "geo"))
 		opt <- spatOptions(filename, ...)
 		target <- as.numeric(target[1])
 		keepNA <- FALSE
@@ -59,7 +57,7 @@ setMethod("distance", signature(x="SpatRaster", y="missing"),
 		} else {
 			exclude <- NA
 		}
-		x@pntr <- x@pntr$rastDistance(target, exclude, keepNA, tolower(unit), TRUE, method, opt)
+		x@pntr <- x@pntr$rastDistance(target, exclude, keepNA, tolower(unit), TRUE, method, isTRUE(values), maxdist, opt)	
 		messages(x, "distance")
 	}
 )
@@ -90,21 +88,11 @@ setMethod("gridDist", signature(x="SpatRaster"),
 
 
 setMethod("distance", signature(x="SpatRaster", y="SpatVector"),
-	function(x, y, unit="m", rasterize=FALSE, method="cosine", filename="", ...) {
+	function(x, y, unit="m", rasterize=FALSE, method="haversine", filename="", ...) {
 		opt <- spatOptions(filename, ...)
 		unit <- as.character(unit[1])
-
-		if (!(method %in% c("geo", "haversine", "cosine"))) {
-			error("distance", "not a valid method. Should be one of: 'geo', 'haversine', 'cosine'")
-		}
-
+		method <- match.arg(tolower(method), c("cosine", "haversine", "geo"))
 		x@pntr <- x@pntr$vectDistance(y@pntr, rasterize, unit, method, opt)
-
-#		if (rasterize) {
-#			x@pntr <- x@pntr$vectDistanceRasterize(y@pntr, NA, NA, unit, method, opt)
-#		} else {
-#			x@pntr <- x@pntr$vectDistanceDirect(y@pntr, unit, method, opt)
-#		}
 		messages(x, "distance")
 	}
 )
@@ -139,21 +127,17 @@ mat2wide <- function(m, sym=TRUE, keep=NULL) {
 }
 
 setMethod("distance", signature(x="SpatVector", y="ANY"),
-	function(x, y, sequential=FALSE, pairs=FALSE, symmetrical=TRUE, unit="m", method="geo") {
+	function(x, y, sequential=FALSE, pairs=FALSE, symmetrical=TRUE, unit="m", method="geo", use_nodes=FALSE) {
 		if (!missing(y)) {
 			error("distance", "If 'x' is a SpatVector, 'y' should be a SpatVector or missing")
 		}
-
-		if (!(method %in% c("geo", "haversine", "cosine"))) {
-			error("distance", "not a valid method. Should be one of: 'geo', 'haversine', 'cosine'")
-		}
-
-		if (sequential) {
-			return( x@pntr$distance_self(sequential, unit, method))
-		}
-		unit <- as.character(unit[1])
-		d <- x@pntr$distance_self(sequential, unit, method)
+		method <- match.arg(tolower(method), c("cosine", "haversine", "geo"))
+		opt <- spatOptions()	 
+		d <- x@pntr$distance_self(sequential, unit, method, use_nodes[1], opt)		
 		messages(x, "distance")
+		if (sequential) {
+			return(d)
+		}
 		class(d) <- "dist"
 		attr(d, "Size") <- nrow(x)
 		attr(d, "Diag") <- FALSE
@@ -170,9 +154,11 @@ setMethod("distance", signature(x="SpatVector", y="ANY"),
 
 
 setMethod("distance", signature(x="SpatVector", y="SpatVector"),
-	function(x, y, pairwise=FALSE, unit="m", method = "cosine") {
+	function(x, y, pairwise=FALSE, unit="m", method = "haversine", use_nodes=FALSE) {
 		unit <- as.character(unit[1])
-		d <- x@pntr$distance_other(y@pntr, pairwise, unit, method)
+		method <- match.arg(tolower(method), c("cosine", "haversine", "geo"))
+		opt <- spatOptions()
+		d <- x@pntr$distance_other(y@pntr, pairwise, unit, method, use_nodes[1], opt)
 		messages(x, "distance")
 		if (!pairwise) {
 			d <- matrix(d, nrow=nrow(x), ncol=nrow(y), byrow=TRUE)
