@@ -5,10 +5,10 @@
 
 
 setMethod("init", signature(x="SpatRaster"),
-	function(x, fun, ..., filename="", wopt=list()) {
+	function(x, fun, ..., filename="", overwrite=FALSE, wopt=list()) {
 		x <- rast(x)
 		if (is.character(fun)) {
-			opt <- spatOptions(filename, wopt=wopt)
+			opt <- spatOptions(filename, overwrite=overwrite, wopt=wopt)
 			x <- rast(x, 1)
 			fun <- fun[1]
 			if (fun %in% c("x", "y", "row", "col", "cell", "chess")) {
@@ -21,7 +21,7 @@ setMethod("init", signature(x="SpatRaster"),
 				error("init", "unknown function")
 			}
 		} else if (is.numeric(fun) || is.logical(fun)) {
-			opt <- spatOptions(filename, wopt=wopt)
+			opt <- spatOptions(filename, overwrite=overwrite, wopt=wopt)
 			x@pntr <- x@pntr$initv(fun, opt)
 			messages(x, "init")
 		} else {
