@@ -22,8 +22,10 @@ parfun <- function(cls, d, fun, model, ...) {
 		d <- data.frame(d)
 	}
 	if (!is.null(const)) {
+		nms <- names(const)
 		for (i in 1:ncol(const)) {
-			d <- cbind(d, const[,i,drop=FALSE])
+			# avoid rowname recycling warnings
+			d[[ nms[i] ]] <- const[[ nms[i] ]]
 		}
 	}
 	if (na.rm) {
@@ -65,7 +67,8 @@ parfun <- function(cls, d, fun, model, ...) {
 			r <- fun(model, d, ...)
 		}
 		if (is.list(r)) {
-			r <- as.data.frame(lapply(r, as.numeric))
+			r <- as.data.frame(r)
+			r <- data.frame(lapply(r, as.numeric))
 		} else if (is.factor(r)) {
 			r <- as.integer(r)
 		} else if (is.data.frame(r)) {
@@ -88,6 +91,8 @@ parfun <- function(cls, d, fun, model, ...) {
 	r
 }
 
+#r <- .runModel(model, fun, d, nl, const, na.rm, index, cores=NULL)
+
 
 .getFactors <- function(model, fun, d, nl, const, na.rm, index, ...) {
 	if (!is.data.frame(d)) {
@@ -122,7 +127,7 @@ parfun <- function(cls, d, fun, model, ...) {
 		data.frame(value=1:length(levs), class=levs)
 	} else if (is.list(r) || is.data.frame(r)) {
 		r <- as.data.frame(r)
-		out <- sapply(r, levels)
+		out <- lapply(r, levels)
 		for (i in 1:length(out)) {
 			if (!is.null(out[[i]])) {
 				out[[i]] <- data.frame(value=1:length(out[[i]]), label=out[[i]])
@@ -223,7 +228,10 @@ setMethod("predict", signature(object="SpatRaster"),
 		#tomat <- FALSE
 		readStart(object)
 		on.exit(readStop(object))
-
+		if (!is.null(const)) {
+			const <- data.frame(const)[1,,drop=FALSE]
+			rownames(const) <- NULL
+		}
 		out <- find_dims(object, model, nc, fun, const, na.rm, index, ...)
 		nl <- nlyr(out)
 		
