@@ -158,11 +158,11 @@ setMethod ("show" , "SpatVector",
 		cat(" geometry    :", geomtype(object), "\n")
 		cat(" dimensions  : ", d[1], ", ", d[2], "  (geometries, attributes)\n", sep="" )
 		cat(" extent      : ", e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
-		if (object@ptr$source != "") {
-			if (object@ptr$layer != tools::file_path_sans_ext(win_basename(object@ptr$source))) {
-				cat(" source      : ", win_basename(object@ptr$source), " (", object@ptr$layer, ")\n", sep="")
+		if (object@pntr$source != "") {
+			if (object@pntr$layer != tools::file_path_sans_ext(win_basename(object@pntr$source))) {
+				cat(" source      : ", win_basename(object@pntr$source), " (", object@pntr$layer, ")\n", sep="")
 			} else {
-				cat(" source      : ", win_basename(object@ptr$source), "\n", sep="")
+				cat(" source      : ", win_basename(object@pntr$source), "\n", sep="")
 			}
 		}
 		cat(" coord. ref. :", .name_or_proj4(object), "\n")
@@ -183,10 +183,10 @@ setMethod ("show" , "SpatVectorProxy",
 		cat(" geometry    :", geomtype(object), "\n")
 		cat(" dimensions  : ", d[1], ", ", d[2], "  (geometries, attributes)\n", sep="" )
 		cat(" extent      : ", e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
-		if (object@ptr$v$layer != tools::file_path_sans_ext(win_basename(object@ptr$v$source))) {
-			cat(" source      : ", win_basename(object@ptr$v$source), " (", object@ptr$v$layer, ")\n", sep="")
+		if (object@pntr$v$layer != tools::file_path_sans_ext(win_basename(object@pntr$v$source))) {
+			cat(" source      : ", win_basename(object@pntr$v$source), " (", object@pntr$v$layer, ")\n", sep="")
 		} else {
-			cat(" source      : ", win_basename(object@ptr$v$source), "\n", sep="")
+			cat(" source      : ", win_basename(object@pntr$v$source), "\n", sep="")
 		}
 		cat(" coord. ref. :", .name_or_proj4(object), "\n")
 		dd <- get.data.frame(object)
@@ -214,7 +214,7 @@ setMethod ("show" , "SpatRaster",
 				txt <- "extent (win): "
 			}
 			cat(txt, w[1], ", ", w[2], ", ", w[3], ", ", w[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
-			#e <- as.vector(object@ptr$source[[1]]$window$full_extent$vector)
+			#e <- as.vector(object@pntr$source[[1]]$window$full_extent$vector)
 			#cat("full extent : " , e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
 		} else {
 			e <- as.vector(ext(object))
@@ -292,13 +292,13 @@ setMethod ("show" , "SpatRaster",
 					cat("source      :", sources[1], "\n")
 				}
 			}
-			rgbtype <- object@ptr$rgbtype
+			rgbtype <- object@pntr$rgbtype
 			if (rgbtype != "") {
 				rdgb <- RGB(object)
 				if (is.null(rdgb)) rdgb <- 1:3
-				cat(paste("colors", toupper(object@ptr$rgbtype), " :"), paste(rdgb, collapse=", "), "\n")
+				cat(paste("colors", toupper(object@pntr$rgbtype), " :"), paste(rdgb, collapse=", "), "\n")
 			}
-			hasct <- object@ptr$hasColors()
+			hasct <- object@pntr$hasColors()
 			if (any(hasct)) {
 				cat("color table :", paste(which(hasct), collapse=", "), "\n")
 			}
@@ -334,7 +334,7 @@ setMethod ("show" , "SpatRaster",
 			isB <- is.bool(object)
 			if (any(hMM) || any(is.factor(object))) {
 				#r <- minmax(object)
-				rr <- r <- rbind(object@ptr$range_min, object@ptr$range_max)
+				rr <- r <- rbind(object@pntr$range_min, object@pntr$range_max)
 				r[,!hMM] <- c(Inf, -Inf)
 				#sc <- scoff(object)
 				#r <- r * sc[,1] + sc[,2]
@@ -423,11 +423,38 @@ setMethod ("show" , "SpatRaster",
 			}
 
 		}
+		if (object@pntr$hasDepth) {
+			dname <- depthName(object)
+			dunit <- depthUnit(object)
+			if (dname == "depth") {
+				if (dunit == "") {
+					dname <- ""
+				} else {
+					dname <- paste0("[", dunit, "]: ") 
+				}
+			} else {
+				if ((dunit == "") || (dunit == "unknown")) {
+					dname <- paste0(dname, ": ")
+				} else {
+					dname <- paste0(dname, " [", dunit, "]: ")
+				}
+			}
+			
+			label <- "depth       : "
+			dpth <- unique(depth(object))
+			if (length(dpth) > 1) {
+				rd <- range(dpth)
+				dpth <- paste0(label, paste(rd, collapse=" to "), " (", dname, length(dpth), " steps)")
+			} else {
+				dpth <- paste0(label, dpth)			
+			}
+			cat(dpth, "\n")		
+		}
 
-		if (object@ptr$hasTime) {
+		if (object@pntr$hasTime) {
 			label <- "time        "
 			rtim <- range(time(object))
-			tims <- object@ptr$timestep
+			tims <- object@pntr$timestep
 			if (tims == "yearmonths") {
 				rtim <- format_ym(rtim)
 				label <- "time (ymnts)"
@@ -443,7 +470,7 @@ setMethod ("show" , "SpatRaster",
 			}
 			utim <- unique(rtim)
 			if (length(utim) > 1) {
-				ptim <- paste0(label, ": ", paste(rtim, collapse=" to "))
+				ptim <- paste0(label, ": ", paste(rtim, collapse=" to "), " (", length(utim), " steps)")
 			} else {
 				ptim <- paste0(label, ": ", as.character(utim))
 			}
@@ -543,7 +570,7 @@ setMethod("show" , "SpatRasterCollection",
 		cat("extent      : " , e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
 		
 		
-		crs <- .name_or_proj4(object@ptr$x[[1]])
+		crs <- .name_or_proj4(object@pntr$x[[1]])
 		if (crs != "") cat("crs (first) :", crs,	 "\n")
 		ln <- names(object)
 		if (any(ln != "")) {
@@ -560,7 +587,7 @@ setMethod("show" , "SpatGraticule",
 	function(object) {
 		cat("class       :" , class(object), "\n")
 		v <- vect()
-		v@ptr <- object@ptr
+		v@pntr <- object@pntr
 		cat("lon         :" , stats::na.omit(v$lon), "\n")		
 		cat("lat         :" , stats::na.omit(v$lat), "\n")		
 		cat("coord. ref. :", .name_or_proj4(v), "\n")
