@@ -39,10 +39,19 @@
 #endif
 
 
+
+// [[Rcpp::export(name = ".have_TBB")]]
+bool have_TBB() {
+	#ifdef HAVE_TBB
+		return true;
+	#else 
+		return false;
+	#endif 
+}
+
+
 //from sf
-
 #ifdef projh
-
 // [[Rcpp::export]]
 std::string proj_version() {
 	std::stringstream buffer;
@@ -271,7 +280,7 @@ std::vector<std::vector<std::string>> gdal_drivers() {
     GDALDriver *poDriver;
     char **papszMetadata;
 	for (size_t i=0; i<n; i++) {
-	    poDriver = GetGDALDriverManager()->GetDriver(i);
+	    poDriver = GetGDALDriverManager()->GetDriver((int)i);
 		const char* ss = poDriver->GetDescription();
 		if (ss != NULL ) s[0][i] = ss;
 		ss = poDriver->GetMetadataItem( GDAL_DMD_LONGNAME );
@@ -446,7 +455,7 @@ std::vector<double> percRank(std::vector<double> x, std::vector<double> y, doubl
 					break;
 				}
 			}
-			double z = (b + 0.5 * t) / nx;
+			double z = ((double)b + 0.5 * (double)t) / (double) nx;
 			if (tail == 1) { // both
 				if (z > 0.5) {
 					z = 2 * (1 - z);
@@ -473,16 +482,43 @@ std::vector<double> percRank(std::vector<double> x, std::vector<double> y, doubl
 }
 
 
+// [[Rcpp::export(name = ".clearVSIcache")]]
+void clearVSIcache(bool vsi) {
+	//if (vsi)
+	VSICurlClearCache();
+}
+
 // [[Rcpp::export(name = ".setGDALCacheSizeMB")]]
-void setGDALCacheSizeMB(double x) {
-  GDALSetCacheMax64(static_cast<int64_t>(x) * 1024 * 1024);
+void setGDALCacheSizeMB(double x, bool vsi) {
+	if (vsi) {
+		int64_t v = int64_t(x * 1024 * 1024);
+		CPLSetConfigOption("CPL_VSIL_CURL_CACHE_SIZE", std::to_string(v).c_str());
+	} else {
+		GDALSetCacheMax64(static_cast<int64_t>(x) * 1024 * 1024);
+	}
+
 }
 
 // [[Rcpp::export(name = ".getGDALCacheSizeMB")]]
-double getGDALCacheSizeMB() {
-  return static_cast<double>(GDALGetCacheMax64() / 1024 / 1024);
+double getGDALCacheSizeMB(bool vsi) {
+	if (vsi) {
+		std::string out = gdal_getconfig("CPLGetConfigOption");
+		Rcpp::Rcout << out << std::endl;
+		if (out == "") return NAN;
+		double v = -1; 
+		try {
+			v = stod(out) / (1024 * 1024);
+		} catch(...){
+			return(NAN);
+		}
+		return(v);
+		
+	} else {
+		return static_cast<double>(GDALGetCacheMax64() / 1024 / 1024);
+	}
 }
 
+
 // convert NULL-terminated array of strings to std::vector<std::string>
 std::vector<std::string> charpp2vect(char **cp) {
 	std::vector<std::string> out;
@@ -530,6 +566,7 @@ bool set_proj_search_paths(std::vector<std::string> paths) {
 #endif
 }
 
+
 // [[Rcpp::export(name = ".PROJ_network")]]
 std::string PROJ_network(bool enable, std::string url) {
 	std::string s = "";
@@ -563,8 +600,8 @@ double pearson_cor(std::vector<double> x, std::vector<double> y, bool narm) {
 		}
 	}
 	size_t n = x.size();
-	double xbar = accumulate(x.begin(), x.end(), 0.0) / n;
-	double ybar = accumulate(y.begin(), y.end(), 0.0) / n;
+	double xbar = accumulate(x.begin(), x.end(), 0.0) / (double)n;
+	double ybar = accumulate(y.begin(), y.end(), 0.0) / (double)n;
 	double numer = 0;
 	for (size_t i=0; i<n; i++) {
 		numer += (x[i]-xbar) * (y[i]-ybar);
@@ -627,7 +664,7 @@ Rcpp::IntegerMatrix uniqueSymmetricRows(std::vector<size_t> x, std::vector<size_
 	size_t n = x.size();
 	for (size_t i=0; i<n; i++) {
 		if (x[i] > y[i]) {
-			double tmp = x[i];
+			size_t tmp = x[i];
 			x[i] = y[i];
 			y[i] = tmp;
 		}
