@@ -279,8 +279,8 @@ std::vector<std::vector<std::string>> sdinfo(std::string fname) {
 			name.push_back(s);
 			std::string vdelim = ":";
 			size_t pos = s.find_last_of(vdelim);
-			if (sub.constructFromFile(s, {-1}, {""}, {}, {}, false)) {
-				nr.push_back( std::to_string(sub.nrow()));
+			if (sub.constructFromFile(s, {-1}, {""}, {}, {}, false, {})) {
+				nr.push_back(std::to_string(sub.nrow()));
 				nc.push_back(std::to_string(sub.ncol()));
 				nl.push_back(std::to_string(sub.nlyr()));
 			}
