@@ -18,6 +18,7 @@
 #include "spatRaster.h"
 #include "math_utils.h"
 #include "string_utils.h"
+
 #include "file_utils.h"
 #include "vecmath.h"
 #include "recycle.h"
@@ -287,6 +288,9 @@ bool checkFormatRequirements(const std::string &driver, std::string &filename, s
 			msg = "SAGA filenames must end on '.sdat'";
 			return false;
 		}
+	} else if (driver == "VRT") {
+		msg = "Cannot directly write to VRT (see '?vrt')";
+		return false;
 	}
 
 	return true;
@@ -614,14 +618,14 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 
 	bool scoffwarning = false;
 
-	if (driver == "GTiff") {
-		std::vector<std::string> m = getTags();
-		if (m.size() > 0) {
-			for (size_t i=0; i<m.size(); i+=2) {
-				poDS->SetMetadataItem(m[i].c_str(), m[i+1].c_str(), "USER_TAGS");
-			}
+	// if (driver == "GTiff") {
+	std::vector<std::vector<std::string>> m = getTags();
+	if (m.size() == 3) {
+		for (size_t i=0; i<m[0].size(); i++) {
+			poDS->SetMetadataItem(m[1][i].c_str(), m[2][i].c_str(), m[0][i].c_str());
 		}
 	}
+	// }
 
 	std::vector<std::string> tstr, ustr;
 	bool wtime = false;
@@ -676,7 +680,7 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 		*/
 		poBand->SetDescription(nms[i].c_str());
 
-		if (driver == "GTiff") {
+		// if (driver == "GTiff") {
 			std::vector<std::string> m = getLyrTags({i});
 			if (m.size() > 0) {
 				for (size_t j=0; j<m.size(); j+=3) {
@@ -694,7 +698,7 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 			if (wunit) {
 				poBand->SetMetadataItem("UNIT", ustr[i].c_str());				
 			}
-		}
+		// }
 
 
 		if ((i==0) || (driver != "GTiff")) {
@@ -843,25 +847,44 @@ void tmp_min_max_na(std::vector<T> &out, const std::vector<double> &v, const dou
 
 
 template <typename Iterator>
-void minmaxlim(Iterator start, Iterator end, double &vmin, double &vmax, const double &lmin, const double &lmax, bool& outrange) {
+void minmaxlim(Iterator start, Iterator end, double &vmin, double &vmax, const double &lmin, const double &lmax, bool& outrange, double exclude=NAN) {
     vmin = std::numeric_limits<double>::max();
     vmax = std::numeric_limits<double>::lowest();
     bool none = true;
-	for (Iterator v = start; v !=end; ++v) {
-		if (!std::isnan(*v)) {
-			if (*v >= lmin && *v <= lmax) {
-				if (*v > vmax) {
-					vmax = *v;
-					none = false;
+	
+	if (!std::isnan(exclude)) {
+		for (Iterator v = start; v !=end; ++v) {
+			if (!std::isnan(*v) && (exclude != *v)) {
+				if (*v >= lmin && *v <= lmax) {
+					if (*v > vmax) {
+						vmax = *v;
+						none = false;
+					}
+					if (*v < vmin) {
+						vmin = *v;
+					}
+				} else {
+					outrange = true;
 				}
-				if (*v < vmin) {
-					vmin = *v;
-				}
-			} else {
-				outrange = true;
 			}
 		}
-    }
+	} else {
+		for (Iterator v = start; v !=end; ++v) {
+			if (!std::isnan(*v)) {
+				if (*v >= lmin && *v <= lmax) {
+					if (*v > vmax) {
+						vmax = *v;
+						none = false;
+					}
+					if (*v < vmin) {
+						vmin = *v;
+					}
+				} else {
+					outrange = true;
+				}
+			}
+		}
+	}
     if (none) {
         vmin = NAN;
         vmax = NAN;
@@ -890,28 +913,32 @@ bool SpatRaster::writeValuesGDAL(std::vector<double> &vals, size_t startrow, siz
 		}
 	}
 
+	int hasNA = 0;
+	double na = source[0].gdalconnection->GetRasterBand(1)->GetNoDataValue(&hasNA);
+	if (!hasNA) na = NAN;
+
 	if ((compute_stats) && (!gdal_stats)) {
 		bool invalid = false;
 		for (size_t i=0; i < nl; i++) {
 			size_t start = nc * i;
 			if (datatype == "INT8S") {
-				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, (double)INT64_MIN, (double)INT64_MAX, invalid);
+				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, (double)INT64_MIN, (double)INT64_MAX, invalid, na);
 			} else if (datatype == "INT4S") {
-				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, (double)INT32_MIN, (double)INT32_MAX, invalid);
+				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, (double)INT32_MIN, (double)INT32_MAX, invalid, na);
 			} else if (datatype == "INT2S") {
-				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, (double)INT16_MIN, (double)INT16_MAX, invalid);
+				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, (double)INT16_MIN, (double)INT16_MAX, invalid, na);
 			} else if (datatype == "INT8U") {
-				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, 0.0, (double)UINT64_MAX, invalid);
+				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, 0.0, (double)UINT64_MAX, invalid, na);
 			} else if (datatype == "INT4U") {
-				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, 0.0, (double)UINT32_MAX, invalid);
+				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, 0.0, (double)UINT32_MAX, invalid, na);
 			} else if (datatype == "INT2U") {
-				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, 0.0, (double)UINT16_MAX, invalid);
+				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, 0.0, (double)UINT16_MAX, invalid, na);
 			} else if (datatype == "INT1U") {
-				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, 0.0, 255.0, invalid);
+				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, 0.0, 255.0, invalid, na);
 			} else if (datatype == "INT1S") {
-				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, -128.0, 127.0, invalid);
+				minmaxlim(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, -128.0, 127.0, invalid, na);
 			} else {
-				minmax(vals.begin()+start, vals.begin()+start+nc, vmin, vmax);
+				minmax(vals.begin()+start, vals.begin()+start+nc, vmin, vmax, na);
 			}
 			if (source[0].has_scale_offset[i]) {
 				vmin = vmin * source[0].scale[i] + source[0].offset[i];
@@ -932,8 +959,6 @@ bool SpatRaster::writeValuesGDAL(std::vector<double> &vals, size_t startrow, siz
 		}
 	}
 
-	int hasNA = 0;
-	double na = source[0].gdalconnection->GetRasterBand(1)->GetNoDataValue(&hasNA);
 	if ((datatype == "FLT8S") || (datatype == "FLT4S")) {
 		if (hasNA) {
 			size_t n = vals.size();
